# coding=utf-8

from typing import List, Dict
from .datasource import QDDataSource
from .embedding import Embedding
from .common import BotPayload, BaseQDBot


class QABot(BaseQDBot):
    def __init__(
        self,
        data_source: QDDataSource,
        embedder: Embedding,
        collection: str,
        score: float,
        limit: int,
    ):
        super(QABot, self).__init__(data_source)
        self.embdder = embedder
        self.collection = collection
        self.score = score
        self.limit = limit

    def formatter(self, line: List) -> Dict:
        # line format: 问题，答案, type, 意图
        return BotPayload(question=line[0], answer=line[1], type=line[2]).to_dict()

    def complete(
        self, prompt: str, score: float = 0, limit: int = 0, stream: bool = False
    ) -> BotPayload:
        _limit = self.limit if limit == 0 else limit
        _score = self.score if score == 0 else score

        points = self.data_source.search(
            prompt,
            _limit,
            embedder=self.embdder,
            collection=self.collection,
        )
        if points is None or len(points) == 0:
            return None

        payloads = [item.payload for item in points if item.score >= _score]

        if len(payloads) == 0:
            return None

        return BotPayload.factory(**payloads[0])
